

proc ch {psfPrefix dcdPrefix xstPrefix selText timestep cutOff outPut} {

    # Input:
    set psf $psfPrefix.psf
    set dcd $dcdPrefix.dcd

    # load trajectory and wait until loading completes 
    mol load psf $psf dcd $dcd
    set n [ molinfo top get numframes ]
    puts "  Loaded $n frames."
    
    # define first and last frames and stride
    set firstframe 0
    set lastframe $n
    set stride 1
   
    # set output file
    set out [open $outPut a]

    # Get the time in nanoseconds.
    set t ""
    set inStream [open ${xstPrefix}.xst r]
    gets $inStream;  gets $inStream;
    while {[gets $inStream line] > 0} {
        lappend t [expr [lindex $line 0] * $timestep * 1.0e-6]
    }
 
    # proceed frame by frame
    for {set i $firstframe} {$i <= $lastframe} {set i [ expr $i + $stride ] } {
        
        set sel [atomselect top "within $cutOff of $selText" frame $i]

        set charge [measure sumweights $sel weight charge] 
        puts $out "[lindex $t $i]\t$charge"
        
        $sel delete
        
        puts "frame $i" 
    }

    mol delete top

    close $out

}


if {$argc < 7} {
    puts "vmd -dispdev text -e $argv0 -args psfPrefix dcdPrefix xstPrefix selText timestep cutOff outPut"
    exit
}

set psfPrefix [lindex $argv 0]
set dcdPrefix [lindex $argv 1]
set xstPrefix [lindex $argv 2]
set selText [lindex $argv 3]
set timestep [lindex $argv 4]
set cutOff [lindex $argv 5]
set outPut [lindex $argv 6]

ch $psfPrefix $dcdPrefix $xstPrefix $selText $timestep $cutOff $outPut

exit



